<?php
/**
 * Haraiso - the PHP Validation Framework
 *
 * PHP version 5
 *
 * @category  Haraiso
 * @package   Haraiso
 * @link      http://bitbucket.org/localdisk/haraiso/
 * @author    MATSUO Masaru
 * @copyright 2010 MATSUO Masaru
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 *
 */
require_once 'AnnotationReader.php';
require_once 'validator/ValidatorFactory.php';
require_once 'validator/AbstractValidator.php';

/**
 * Haraiso
 *
 * @package   Haraiso
 * @author    MATSUO Masaru
 */
class Haraiso {

    /**
     * validator name cache
     *
     * @var array
     */
    private static $_validators = array();

    /**
     * Constructor
     */
    public function __construct() {
        self::load();
    }

    /**
     * validate
     *
     * @param mixed  $obj
     * @param string $callback
     */
    public function validate($obj, $callback = null) {
        if ($callback !== null) {
            if (is_callable(array($obj, $callback)) === false) {
                throw new Exception("callback method:$callback is not callable.");
            }
        }
        $clazz = new ReflectionClass($obj);
        $fields = $clazz->getProperties();
        $messages = array();
        foreach ($fields as $field) {
            $annotations = AnnotationReader::getAnnotation($field, self::$_validators);
            $val = $this->_getProperty($field, $clazz, $obj);
            foreach ($annotations as $annotation) {
                $validator = ValidatorFactory::create($annotation, self::$_validators);
                $ret = $validator->validate($val, $field->name);
                if ($ret !== true) $messages[] = $ret;
            }
        }
        if ($callback === null) return $messages;
        return call_user_func(array($obj, $callback), $messages);
    }

    /**
     * load
     *
     * @return void
     */
    private static function load() {
        if (count(self::$_validators) !== 0) return;
        $files = glob(dirname(__FILE__) . '/validator/impl/' . '*.php');
        foreach ($files as $file) {
            require_once $file;
            $name = basename($file);
            $name = substr($name, 0, strrpos($name, '.'));
            if (!class_exists($name)) throw new Exception("class:$name is not found");
            self::$_validators[] = $name;
        }
    }

    /**
     * getProperty
     *
     * @param  ReflectionProperty $property
     * @param  ReflectionClass $clazz
     * @param  mixed $obj
     * @return mixed
     */
    private function _getProperty(ReflectionProperty $property, ReflectionClass $clazz, $obj) {
        $name = $property->name;
        if ($property->isPublic() === true) {
            return $property->getValue($obj);
        }
        $methodName = 'get' . ucfirst($name);
        if (is_callable(array($obj, $methodName)) === true) {
            $method = $clazz->getMethod($methodName);
            return $method->invoke($obj);
        }
        $methodName = '__get';
        if (is_callable(array($obj, $methodName)) === true) {
            $method = $clazz->getMethod($methodName);
            return $obj->$name;
        }
        return null;
    }
}