<?php
/**
 * Haraiso - the PHP Validation Framework
 *
 * PHP version 5
 *
 * @category  Haraiso
 * @package   Haraiso
 * @link      http://bitbucket.org/localdisk/haraiso/
 * @author    MATSUO Masaru
 * @copyright 2010 MATSUO Masaru
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 *
 */
require_once dirname(__FILE__) . '/../Message.php';

/**
 * AbstractValidator
 *
 * @package   Haraiso/validate
 * @author    MATSUO Masaru
 */
abstract class AbstractValidator {

    /**
     * args
     *
     * @var array
     */
    protected $_args;
    /**
     * key
     * 
     * @var string
     */
    protected $_key;
    /**
     * messages
     *
     * @var array
     */
    private static $_messages = array();
    /**
     * locale
     *
     * @var string
     */
    private static $_locale = null;

    /**
     * Constructor
     */
    public function __construct() {
        self::_setLocale();
        self::_setMessages();
    }

    /**
     * __set
     *
     * @param string $name
     * @param mixed  $value
     */
    public function __set($name, $value) {
        $class_name = get_class($this);
        if (property_exists($class_name, $name) === true) {
            $this->$name = $value;
        }
        if ($name === '_args') {
            $this->_key = array_key_exists('key', $this->_args)
                    ? $this->_args['key'] : strtolower($class_name);
        }
    }

    /**
     * getMessage
     *
     * @param  string $key
     * @param  array  $args
     * @return string
     */
    public function getMessage($key, $args) {
        $message = self::$_messages[$key];
        if ($message === null) $message = "$key is not found.";
        return vsprintf($message, $args);
    }

    /**
     * setLocale
     */
    private static function _setLocale() {
        if (self::$_locale === null) {
            $language = getenv('HTTP_ACCEPT_LANGUAGE');
            if ($language !== false && empty($language) !== false) {
                self::$_locale = array_shift(preg_split('/,\s*/', $language));
            } else {
                self::$_locale = 'ja';
            }
        }
    }

    /**
     * setMessages
     */
    private static function _setMessages() {
        if (count(self::$_messages) === 0) {
            $ini = dirname(__FILE__) . '/../resource/messages_' . self::$_locale . '.ini';
            self::$_messages = parse_ini_file($ini);
            if (self::$_messages === false) throw new Exception("$ini not found.");
        }
    }

    /**
     * validate
     *
     * @param  mixed  $val
     * @param  string $name
     * @return bool|string
     */
    abstract public function validate($val, $name);
}